@@ -1165,13 +1165,13 @@ class SphinxRenderer:
 
         # Get all sub sections
         for sectiondef in node.sectiondef:
+            kind = sectiondef.kind
+            if section_order is not None and kind not in section_order:
+                continue
             child_nodes = self.render(sectiondef)
             if not child_nodes:
                 # Skip empty section
                 continue
-            kind = sectiondef.kind
-            if section_order is not None and kind not in section_order:
-                continue
             rst_node = nodes.container(classes=['breathe-sectiondef'])
             rst_node.document = self.state.document
             rst_node['objtype'] = kind
@@ -1606,12 +1606,14 @@ class SphinxRenderer:
             if dom == 'py':
                 declaration = name + node.get_argsstring()
             else:
-                declaration = ' '.join([
-                    self.create_template_prefix(node),
-                    ''.join(n.astext() for n in self.render(node.get_type())),  # type: ignore
-                    name,
-                    node.get_argsstring()
-                ])
+                elements = [self.create_template_prefix(node)]
+                if node.kind == 'friend':
+                    elements.append('friend')
+                elements.append(''.join(n.astext()
+                                        for n in self.render(node.get_type())))  # type: ignore
+                elements.append(name)
+                elements.append(node.get_argsstring())
+                declaration = ' '.join(elements)
             nodes = self.handle_declaration(node, declaration)
             return nodes
         else:
@@ -1781,7 +1783,11 @@ class SphinxRenderer:
         desc += signode
 
         typ = ''.join(n.astext() for n in self.render(node.get_type()))  # type: ignore
-        assert typ in ("friend class", "friend struct")
+        # in Doxygen < 1.9 the 'friend' part is there, but afterwards not
+        # https://github.com/michaeljones/breathe/issues/616
+        assert typ in ("friend class", "friend struct", "class", "struct")
+        if not typ.startswith('friend '):
+            typ = 'friend ' + typ
         signode += addnodes.desc_annotation(typ, typ)
         signode += nodes.Text(' ')
         # expr = cpp.CPPExprRole(asCode=False)
